package eu.mhsl.minenet.minigames.instance.game.stateless.types.tetris.game;

import eu.mhsl.minenet.minigames.instance.game.stateless.StatelessGame;
import net.minestom.server.MinecraftServer;
import net.minestom.server.coordinate.Pos;
import net.minestom.server.timer.Scheduler;
import net.minestom.server.timer.TaskSchedule;

import java.util.Random;

public class TetrisGame {
    private final StatelessGame instance;
    private final Playfield playfield;
    private int speed = 1;
    private int lines = 0;
    public Tetromino currentTetromino;
    private Tetromino nextTetromino;
    private final Random random = new Random();

    private final Pos tetrominoSpawnPosition;

    public TetrisGame(StatelessGame instance, Pos lowerLeftCorner) {
        this(instance, lowerLeftCorner, new Tetromino(instance, Tetromino.Shape.J), new Tetromino(instance, Tetromino.Shape.T));
    }

    public TetrisGame(StatelessGame instance, Pos lowerLeftCorner, Tetromino startTetromino, Tetromino nextTetromino) {
        this.playfield = new Playfield(lowerLeftCorner);
        this.currentTetromino = startTetromino;
        this.nextTetromino = nextTetromino;
        this.instance = instance;

        this.tetrominoSpawnPosition = this.playfield.getTetrominoSpawnPosition();
    }

    public void start() {
        Scheduler scheduler = MinecraftServer.getSchedulerManager();
        scheduler.submitTask(() -> {
            this.tick();
            return TaskSchedule.tick(40/this.speed);
        });
    }

    public void generate() {
        this.playfield.generate(this.instance);

        this.currentTetromino.setPosition(this.tetrominoSpawnPosition);
        this.currentTetromino.draw();
    }

    public void tick() {
        if(!currentTetromino.moveDown()) {
            currentTetromino = nextTetromino;
            currentTetromino.setPosition(this.tetrominoSpawnPosition);
            currentTetromino.draw();
            nextTetromino = getNextTetromino();
        }
    }


    private Tetromino getNextTetromino() {
        int randomNumber = random.nextInt(1, 7);
        Tetromino.Shape nextShape;

        switch (randomNumber) {
            case 2 -> nextShape = Tetromino.Shape.J;
            case 3 -> nextShape = Tetromino.Shape.L;
            case 4 -> nextShape = Tetromino.Shape.O;
            case 5 -> nextShape = Tetromino.Shape.S;
            case 6 -> nextShape = Tetromino.Shape.T;
            case 7 -> nextShape = Tetromino.Shape.Z;
            default -> nextShape = Tetromino.Shape.I;
        }

        return new Tetromino(this.instance, nextShape);
    }
}
